/*-
 * ============LICENSE_START=======================================================
 *  Copyright (C) 2022-2023 Nordix Foundation.
 * ================================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 * ============LICENSE_END=========================================================
 */

package org.oransc.nonrtric.plt.capifcore.controller;

import java.util.ArrayList;
import java.util.List;

import org.oransc.capif.discoverservice.server.api.AllServiceAPIsApi;
import org.oransc.capif.model.AefProfile;
import org.oransc.capif.model.CommunicationType;
import org.oransc.capif.model.DataFormat;
import org.oransc.capif.model.DiscoveredAPIs;
import org.oransc.capif.model.Protocol;
import org.oransc.capif.model.ServiceAPIDescription;
import org.oransc.capif.model.Version;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

@RestController
@RequestMapping("/service-apis/v1")
public class DiscoverService implements AllServiceAPIsApi {

    ApiRegister apiRegister;

    public DiscoverService(ApiRegister apiRegister) {
        this.apiRegister = apiRegister;


    }

    @Override
    public ResponseEntity<DiscoveredAPIs> allServiceAPIsGet(String apiInvokerId, String apiName, String apiVersion,
            CommunicationType commType, Protocol protocol, String aefId, DataFormat dataFormat, String apiCat,
            String supportedFeatures, String apiSupportedFeatures) {
        if (apiInvokerId == null || apiInvokerId.isBlank()) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Api Invoker Id must be provided");
        }

        DiscoveredAPIs discoveredApis = new DiscoveredAPIs();
        ServiceAPIDescription[] allApis = apiRegister.getAPIs();
        ArrayList<ServiceAPIDescription> filteredApis = new ArrayList<ServiceAPIDescription>();
        String gatewayDomain = "r1-expo-func-aef";

        for (ServiceAPIDescription serviceAPIDescription : allApis) {
            List<AefProfile> profiles = serviceAPIDescription.getAefProfiles();
            if (!matchesFilter(serviceAPIDescription, profiles, apiName, apiVersion))
                continue;

            for (AefProfile profile : profiles) {
                profile.setDomainName(gatewayDomain);

            }
            filteredApis.add(serviceAPIDescription);

        }
        discoveredApis.setServiceAPIDescriptions(filteredApis);

        return ResponseEntity.ok(discoveredApis);


    }

    public boolean matchesFilter(ServiceAPIDescription serviceAPIDescription, List<AefProfile> profiles, String apiName,
            String apiVersion) {
        if (apiName != null && !serviceAPIDescription.getApiName().equals(apiName))
            return false;
        boolean match = false;


        for (AefProfile profile : profiles) {

            if (apiVersion != null) {
                for (Version ver : profile.getVersions()) {
                    if (ver.getApiVersion().equals(apiVersion))
                        match = true;
                }

            } else
                match = true;
        }


        return match;

    }



}
