// -
//   ========================LICENSE_START=================================
//   O-RAN-SC
//   %%
//   Copyright (C) 2022: Nordix Foundation
//   %%
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//        http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//   ========================LICENSE_END===================================
//

package org.oransc.nonrtric.plt.capifcore.controller;

import java.net.URI;
import java.util.HashMap;
import java.util.Map;

import org.oransc.capif.model.AefProfile;
import org.oransc.capif.model.ServiceAPIDescription;
import org.oransc.capif.publishservice.server.api.ApfIdApi;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

@RestController
@RequestMapping("/published-apis/v1")
public class PublishService implements ApfIdApi, ApiRegister {
    private final Map<String, ServiceAPIDescription> publishedServices = new HashMap<>();

    @Override
    public ResponseEntity<ServiceAPIDescription> apfIdServiceApisPost(String apfId,
        ServiceAPIDescription serviceAPIDescription) {
        serviceAPIDescription.apiId("api_id_" + serviceAPIDescription.getApiName());
        synchronized (publishedServices) {
            publishedServices.put(serviceAPIDescription.getApiId(), serviceAPIDescription);
        }
        URI uri = ServletUriComponentsBuilder.fromCurrentRequest().path("/{id}")
            .buildAndExpand(serviceAPIDescription.getApiId()).toUri();
        return ResponseEntity.created(uri).body(serviceAPIDescription);
    }

    @Override
    public ResponseEntity<ServiceAPIDescription> apfIdServiceApisServiceApiIdGet(String serviceApiId, String apfId) {
        synchronized (publishedServices) {
            if (publishedServices.containsKey(serviceApiId)) {
                return ResponseEntity.ok(publishedServices.get(serviceApiId));
            }
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, "The requested service is not published.");
        }
    }

    @Override
    public boolean areAPIsRegistered(ServiceAPIDescription[] serviceDescriptions) {
        return false;
    }

    @Override
    public ServiceAPIDescription[] getAPIs() {
        synchronized (publishedServices) {
            return publishedServices.values().toArray(new ServiceAPIDescription[0]);
        }
    }

    @Override
    public boolean isAPIRegistered(String aefId, String apiName) {
        synchronized (publishedServices) {
            boolean registered = false;
            for (ServiceAPIDescription api : publishedServices.values()) {
                if (apiName.equals(api.getApiName())) {
                    if (isProfileRegistered(aefId, api))
                        registered = true;
                    break;
                }
            }
            return registered;
        }
    }

    private boolean isProfileRegistered(String aefId, ServiceAPIDescription api) {
        synchronized (publishedServices) {
            boolean registered = false;
            for (AefProfile aefProfile : api.getAefProfiles()) {
                if (aefId.equals(aefProfile.getAefId())) {
                    registered = true;
                    break;
                }
            }
            return registered;
        }
    }
}
