// -
//   ========================LICENSE_START=================================
//   O-RAN-SC
//   %%
//   Copyright (C) 2022: Nordix Foundation
//   %%
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//        http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//   ========================LICENSE_END===================================
//

package org.oransc.nonrtric.plt.capifcore;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.junit.jupiter.api.Test;
import org.oransc.capif.model.APIInvokerEnrolmentDetails;
import org.oransc.capif.model.APIProviderEnrolmentDetails;
import org.oransc.capif.model.ServiceAPIDescription;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc
class CapifCoreApplicationTests {
    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private ObjectMapper objectMapper;

    @Test
    void servicesRegistered() throws Exception {
        APIProviderEnrolmentDetails newProvider = new APIProviderEnrolmentDetails();
        newProvider.apiProvDomInfo("domainInfo");
        mockMvc.perform(post("/api-provider-management/v1/registrations").contentType(MediaType.APPLICATION_JSON)
            .content(objectMapper.writeValueAsString(newProvider))).andExpect(status().isCreated());

        ServiceAPIDescription newServiceApi = new ServiceAPIDescription();
        newServiceApi.apiName("apiName");

        mockMvc.perform(post("/published-apis/v1/apfId/service-apis").contentType(MediaType.APPLICATION_JSON)
            .content(objectMapper.writeValueAsString(newServiceApi))).andExpect(status().isCreated());

        APIInvokerEnrolmentDetails newInvoker = new APIInvokerEnrolmentDetails();
        newInvoker.apiInvokerInformation("invokerInfo");
        mockMvc.perform(post("/api-invoker-management/v1/onboardedInvokers").contentType(MediaType.APPLICATION_JSON)
            .content(objectMapper.writeValueAsString(newInvoker))).andExpect(status().isCreated());
    }

}
