/*-
 * ============LICENSE_START=======================================================
 *  Copyright (C) 2022-2023 Nordix Foundation.
 * ================================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 * ============LICENSE_END=========================================================
 */

package org.oransc.nonrtric.plt.capifcore.controller;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.Mockito.when;

import java.util.ArrayList;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.oransc.capif.model.AefProfile;
import org.oransc.capif.model.DiscoveredAPIs;
import org.oransc.capif.model.ServiceAPIDescription;
import org.oransc.capif.model.Version;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.web.server.ResponseStatusException;

@ExtendWith(SpringExtension.class)
@ExtendWith(MockitoExtension.class)
class DiscoverServiceTest {


    DiscoverService testService;

    @Test
    @DisplayName("Get All the Service APIs")
    void testGetAllServiceAPIs(@Mock ServiceAPIDescription serviceAPIDescriptionMock, @Mock ApiRegister apiRegisterMock,
            @Mock AefProfile profileMock) {
        testService = new DiscoverService(apiRegisterMock);
        ServiceAPIDescription[] resultApis = {serviceAPIDescriptionMock};
        ArrayList<AefProfile> profile = new ArrayList<AefProfile>();
        profile.add(profileMock);
        when(apiRegisterMock.getAPIs()).thenReturn(resultApis);
        when(serviceAPIDescriptionMock.getAefProfiles()).thenReturn(profile);
        ResponseEntity<DiscoveredAPIs> allServiceAPIsGetTest =
                testService.allServiceAPIsGet("apiInvokerId ", null, null, null, null, null, null, null, null, null);
        assertThat(allServiceAPIsGetTest.getBody().getServiceAPIDescriptions()).hasSize(1);

    }

    @Test
    @DisplayName("Api Invoker Id Missing With Exception")
    void testApiInvokerIdMissingShouldThrowException(@Mock ApiRegister apiRegisterMock) {
        testService = new DiscoverService(apiRegisterMock);
        ResponseStatusException exception = assertThrows(ResponseStatusException.class, () -> {
            testService.allServiceAPIsGet(" ", null, null, null, null, null, null, null, null, null);
        });
        String actualMessage = "Api Invoker Id must be provided";
        assertTrue(exception.getMessage().contains(actualMessage));
        assertThat(exception.getStatus()).isEqualTo(HttpStatus.BAD_REQUEST);

    }

    @Test
    @DisplayName("Filter criteria based on the apiName, ApiVersion and AefProfile")
    void testFilterCriteria(@Mock ServiceAPIDescription serviceAPIDescriptionMock, @Mock ApiRegister apiRegisterMock,
            @Mock AefProfile profileMock, @Mock ServiceAPIDescription serviceAPIDescription2Mock,
            @Mock Version versionMock) {

        testService = new DiscoverService(apiRegisterMock);
        String apiName = "apiName";
        String apiName_wrong = "Wrong_name";
        String apiVersion = "apiVersion";

        when(serviceAPIDescriptionMock.getApiName()).thenReturn(apiName);
        when(serviceAPIDescription2Mock.getApiName()).thenReturn(apiName_wrong);

        ArrayList<AefProfile> profile = new ArrayList<AefProfile>();
        profile.add(profileMock);
        ArrayList<Version> versionList = new ArrayList<Version>();
        versionList.add(versionMock);

        ServiceAPIDescription[] resultApis = {serviceAPIDescriptionMock, serviceAPIDescription2Mock};
        when(apiRegisterMock.getAPIs()).thenReturn(resultApis);
        when(serviceAPIDescriptionMock.getAefProfiles()).thenReturn(profile);
        when(serviceAPIDescription2Mock.getAefProfiles()).thenReturn(profile);
        when(profileMock.getVersions()).thenReturn(versionList);
        when(versionMock.getApiVersion()).thenReturn(apiVersion);

        ResponseEntity<DiscoveredAPIs> allServiceAPIsGetTest = testService.allServiceAPIsGet("apiInvokerId ", apiName,
                apiVersion, null, null, null, null, null, null, null);
        assertThat(allServiceAPIsGetTest.getBody().getServiceAPIDescriptions()).hasSize(1);


    }



}
