// -
//   ========================LICENSE_START=================================
//   O-RAN-SC
//   %%
//   Copyright (C) 2022: Nordix Foundation
//   %%
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//        http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//   ========================LICENSE_END===================================
//

package org.oransc.nonrtric.plt.capifcore.controller;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertThrows;

import java.net.URI;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.oransc.capif.model.AefProfile;
import org.oransc.capif.model.ServiceAPIDescription;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.server.ResponseStatusException;

@ExtendWith(SpringExtension.class)
class PublishServiceTest {
    // ApiRegister apiRegisterMock = mock(ApiRegister.class);
    @Test
    void testPostNewServiceShouldGetApfIdAndResponseShouldIncludeLocationHeader() throws Exception {
        MockHttpServletRequest request = new MockHttpServletRequest();
        RequestContextHolder.setRequestAttributes(new ServletRequestAttributes(request));

        PublishService apiUnderTest = new PublishService();
        ServiceAPIDescription newServiceApi = new ServiceAPIDescription();
        newServiceApi.apiName("apiName");
        ResponseEntity<ServiceAPIDescription> response = apiUnderTest.apfIdServiceApisPost("apfId", newServiceApi);
        assertThat(response.getStatusCode()).isEqualTo(HttpStatus.CREATED);
        assertThat(response.getBody().getApiId()).isEqualTo("api_id_apiName");
        assertThat(response.getHeaders().getLocation()).isEqualTo(new URI("http://localhost/api_id_apiName"));
    }

    @Test
    void testPostNewServiceShouldBeStored() {
        MockHttpServletRequest request = new MockHttpServletRequest();
        RequestContextHolder.setRequestAttributes(new ServletRequestAttributes(request));

        PublishService apiUnderTest = new PublishService();
        ServiceAPIDescription newServiceAPI = new ServiceAPIDescription();
        String apiName = "apiName";
        newServiceAPI.apiName(apiName);
        AefProfile aefProfile = new AefProfile();
        String aefId = "aefId";
        aefProfile.aefId(aefId);
        newServiceAPI.addAefProfilesItem(aefProfile);
        ResponseEntity<ServiceAPIDescription> response = apiUnderTest.apfIdServiceApisPost("apfId", newServiceAPI);
        assertThat(response.getStatusCode()).isEqualTo(HttpStatus.CREATED);
        assertThat(apiUnderTest.isAPIRegistered(aefId, apiName)).isTrue();
        assertThat(apiUnderTest.getAPIs()).hasSize(1).contains(newServiceAPI);
        response = apiUnderTest.apfIdServiceApisServiceApiIdGet("api_id_apiName", "apfId");
        assertThat(response.getStatusCode()).isEqualTo(HttpStatus.OK);
        assertThat(response.getBody()).isEqualTo(newServiceAPI);
    }

    @Test
    void testGetPublishedServicesWithNotPublishedShouldThrowException() {
        PublishService apiUnderTest = new PublishService();

        ResponseStatusException exception = assertThrows(ResponseStatusException.class, () -> {
            apiUnderTest.apfIdServiceApisServiceApiIdGet("api_id_apiName", "apfId");
        });

        assertThat(exception.getStatus()).isEqualTo(HttpStatus.NOT_FOUND);
        assertThat(exception.getReason()).isEqualTo("The requested service is not published.");
    }
}
